"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PathHelper_1 = require("../lib/PathHelper");
const ObjectMapperRunner_1 = require("./ObjectMapperRunner");
class ObjectMapper {
    constructor() {
        this.params = {};
        this.map = this.map.bind(this);
        this.addAll = this.addAll.bind(this);
        this.add = this.add.bind(this);
        this.apply = this.apply.bind(this);
        this.getChildObjectMapper = this.getChildObjectMapper.bind(this);
    }
    map(path, mappers) {
        return this.add(Object.assign({}, mappers, { path }));
    }
    addAll(mappers) {
        Object.keys(mappers).forEach(key => this.add(Object.assign({}, mappers[key], { path: key })));
        return this;
    }
    add(mappers) {
        this.params[mappers.path || ''] = mappers;
        return this;
    }
    getChildObjectMapper(parentPath) {
        const childObjectMapper = new ObjectMapper();
        Object.values(this.params)
            .filter(({ path: childPath }) => PathHelper_1.PathHelper.isParentPath(parentPath, childPath))
            .forEach(childObjectMapper.add);
        return childObjectMapper;
    }
    apply(data) {
        return new ObjectMapperRunner_1.ObjectMapperRunner(this, this.params, data)
            .apply();
    }
    parse(data) {
        return new ObjectMapperRunner_1.ObjectMapperRunner(this, this.params, data)
            .parse();
    }
}
exports.ObjectMapper = ObjectMapper;
//# sourceMappingURL=ObjectMapper.js.map