"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const HelperFunctions_1 = require("../lib/HelperFunctions");
const PathHelper_1 = require("../lib/PathHelper");
const Parser_1 = require("../parsers/Parser");
class ObjectMapperRunner {
    constructor(parentObjectMapper, params, data) {
        this.parentToChildrenReducer = this.parentToChildrenReducer.bind(this);
        this.expandWildcardsReducer = this.expandWildcardsReducer.bind(this);
        this.parentObjectMapper = parentObjectMapper;
        this.paramsObj = params;
        this.data = data;
        this.expandedParamsObj = PathHelper_1.PathHelper.reduceWildcardsInPaths(Object.keys(this.paramsObj), this.expandWildcardsReducer, this.data, {});
        this.fullParamsObj = PathHelper_1.PathHelper.reduceParentsToChildren(Object.keys(this.expandedParamsObj), this.parentToChildrenReducer, _.cloneDeep(this.expandedParamsObj));
    }
    apply() {
        return Object.values(this.fullParamsObj)
            .reduce((acc, params) => {
            const { path, } = params;
            const mapped = this.applyMappers(params);
            const expanded = HelperFunctions_1.safeInsertValueToPath(mapped, path);
            return _.merge(acc, expanded);
        }, {});
    }
    parse() {
        return Parser_1.Parser.parse(this.data, _(this.expandedParamsObj)
            .values()
            .map(item => (Object.assign({}, item, { as: item.path, select: true })))
            .value());
    }
    parentToChildrenReducer(acc, parentPath, childPath) {
        const parentObj = _.get(acc, parentPath);
        const childObj = this.expandedParamsObj[childPath];
        if (parentObj != null) {
            acc[childPath] = Object.assign({}, _.cloneDeep(parentObj), (childObj != null ? childObj : {}), { path: childPath });
        }
        return acc;
    }
    expandWildcardsReducer(acc, parentPath, expandedPath) {
        acc[expandedPath] = Object.assign({}, _.cloneDeep(this.paramsObj[parentPath]), { path: expandedPath });
        return acc;
    }
    applyMappers(params) {
        const { path, def, globalMapper, arrayMapper, objectMapper, primitiveMapper, } = params;
        const value = HelperFunctions_1.safePathGet(this.data, path, def);
        const mappersToApply = [];
        globalMapper && mappersToApply.push(globalMapper);
        if (this.isItemArray(value, path)) {
            arrayMapper && mappersToApply.push(arrayMapper);
        }
        else if (_.isObject(value)) {
            objectMapper && mappersToApply.push(objectMapper);
        }
        else {
            primitiveMapper && mappersToApply.push(primitiveMapper);
        }
        return mappersToApply.reduce((acc, apply) => {
            const mapped = apply(value, path, this.data, this.parentObjectMapper);
            return HelperFunctions_1.mergeObjectWithConcat(acc, mapped);
        }, {});
    }
    isItemArray(value, path) {
        if (_.isArray(value)) {
            return true;
        }
        const allPaths = Object.keys(this.paramsObj);
        return _.some(allPaths, (paramPath) => {
            return paramPath.startsWith(`${path}[`);
        });
    }
}
exports.ObjectMapperRunner = ObjectMapperRunner;
//# sourceMappingURL=ObjectMapperRunner.js.map